/*
MIT License

Copyright (c) 2021 Sulako

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/


#include <digitalWriteFast.h>       //https://github.com/NicksonYap/digitalWriteFast
#include <avdweb_AnalogReadFast.h>  //https://github.com/avandalen/avdweb_AnalogReadFast
#include "fastPort.h"
#include "motor.h"
#include "g25.h"

//Оси
#define PIN_WHEEL A0
#define PIN_ACC A1
#define PIN_BRAKE A2
#define PIN_CLUTCH A3

//матрица кнопок
uint8_t btn_rows[]={4, 5, 6, 7, 8, 11, 12, 13};
uint8_t btn_cols[]={0, 1, 2, 3};

G25_ G25;
FastPort fastPort;
Motor motor;

uint16_t timerFFB;

//отладка
uint16_t timerStat;
uint16_t countLoop,countFFB;
int8_t lastForce=0;
bool showInfo=false;
bool ffbInfo=false;

void setup(){

 uint8_t i;
  
  //настройка выводов
  pinMode(PIN_WHEEL, INPUT);
  pinMode(PIN_ACC, INPUT);
  pinMode(PIN_BRAKE, INPUT);
  pinMode(PIN_CLUTCH, INPUT);

  for(i=0;i<8;i++)
    pinMode(btn_rows[i], INPUT);
  for(i=0;i<4;i++)
    pinMode(btn_cols[i], INPUT_PULLUP);
    
  delay(1000);
  //Границы значений руля
  G25.axes[AXIS_WHEEL]->setRange(35, 920);
  
  //У педалей автокалибровка
  G25.axes[AXIS_ACC]->setRange(analogRead(PIN_ACC));
  G25.axes[AXIS_BRAKE]->setRange(analogRead(PIN_BRAKE));
  G25.axes[AXIS_CLUTCH]->setRange(analogRead(PIN_CLUTCH));
  
  fastPort.begin();
  motor.begin();

  timerFFB=(uint16_t)micros();
  timerStat=(uint16_t)millis();
  
}

void loop(){
  
    //Опрос осей
    G25.axes[AXIS_WHEEL]->setValue(analogReadFast(PIN_WHEEL));
    G25.axes[AXIS_ACC]->setValue(analogReadFast(PIN_ACC));
    G25.axes[AXIS_BRAKE]->setValue(analogReadFast(PIN_BRAKE));
    G25.axes[AXIS_CLUTCH]->setValue(analogReadFast(PIN_CLUTCH));

    //Опрос кнопок
    uint32_t btns=0;
    uint32_t btnBit=1;
    uint8_t row;
    for(row=0;row<8;row++)
    {
      fastPort.pinMode(btn_rows[row], OUTPUT);
      fastPort.digitalWrite(btn_rows[row], 0);
  
      if (!digitalReadFast(btn_cols[0]))
          btns |= btnBit;
      btnBit=btnBit<<1;
      
      if (!digitalReadFast(btn_cols[1]))
          btns |= btnBit;
      btnBit=btnBit<<1;
      
      if (!digitalReadFast(btn_cols[2]))
          btns |= btnBit;
      btnBit=btnBit<<1;
      
      if (!digitalReadFast(btn_cols[3]))
          btns |= btnBit;
      btnBit=btnBit<<1;
          
      fastPort.pinMode(btn_rows[row], INPUT);
    }
    G25.setButtons(btns);

    //передача и прием данных
    G25.update();
    countLoop++;

    //расчет обратной связи
    int8_t force;
    if ((!G25.FFB.fixedLoop) || ((uint16_t)(micros()-timerFFB)>1500))
    {
      timerFFB=micros();
      lastForce=force=G25.getForce();
      motor.setForce(force);
      countFFB++;
    }   

    //отладка
    if (showInfo)
    if ((uint16_t)(millis()-timerStat)>1000)
    {
        timerStat=millis();
        Serial.println(
            " Wheel: "+String(G25.axes[AXIS_WHEEL]->rawValue)+
            " Acc: "+String(G25.axes[AXIS_ACC]->rawValue)+
            " Brake: "+String(G25.axes[AXIS_BRAKE]->rawValue)+
            " Clutch: "+String(G25.axes[AXIS_CLUTCH]->rawValue)+
            " Force: "+String(lastForce)+
            " loops/sec: "+String(countLoop)+
            " FFB/sec: "+String(countFFB)
          );
        countLoop=0;
        countFFB=0;
    }

    String buf;
    if (Serial.available())
    {
        buf=Serial.readString();   
        if (buf.indexOf("info")!=-1)  //Включение-выключение вывода данных по осям, силе ОС и времени обработки в Serial по команде info
        {
          showInfo=!showInfo;
          countLoop=0;
          countFFB=0;
        }
        if (buf.indexOf("ffb")!=-1)  //Включение-выключение вывода информации об ОС по команде ffb
        {
          ffbInfo=!ffbInfo;
          G25.FFB.info();
        }
    }
    
}
