/*
MIT License

Copyright (c) 2021 Sulako

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/


/*
 *  класс HID в Arduino не может принимать данные
 *  поэтому приходится его переписывать
 */

 
#ifndef HIDDEVICE_H
#define HIDDEVICE_H

#include "Arduino.h"
#include "PluggableUSB.h"
#include "HID.h"

typedef struct {
  InterfaceDescriptor hid;
  HIDDescDescriptor desc;
  EndpointDescriptor in;
  EndpointDescriptor out;
} HIDDescriptorInOut;


class HIDDevice : public PluggableUSBModule {
public:
  HIDDevice(const uint8_t* HIDReport_, unsigned int HIDReportLength_, int8_t ReportID_=0);
  bool isAvailable();
  int SendReport(const void* data, int len);
  unsigned int receive();
  unsigned int receive(uint8_t* data, unsigned int len);
private:
  const void* HIDReport;
  unsigned int HIDReportLength;
  uint8_t ReportID=0;
protected:
  uint8_t endpointTypes[2];
  uint8_t protocol;
  uint8_t idle;
  
  int getInterface(uint8_t *interfaceNumber);
  int getDescriptor(USBSetup &setup);
  bool setup(USBSetup &setup);
  uint8_t getShortName(char* name);
};

#endif
